using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace WinForm.Wizard
{
    [Designer(typeof(System.Windows.Forms.Design.ParentControlDesigner)),Browsable(false)]
    public class WizardStep : ContainerControl
    {
        private readonly Button PreviousButton = new Button();
        private readonly Button NextButton = new Button();
        private readonly Button FinishButton = new Button();
        private readonly Button CancelButton = new Button();
        public bool AllowNext = false;
        private bool isLocked = false;

        private WizardStepType stepType = WizardStepType.Step;
        public WizardStepType StepType
        {
            get { return stepType; }
            set { stepType = value; DrawWizardButton(value); }
        }

        public WizardStep()
        {
            InitStepButtons();
            Size = new Size(300,100);
        }

        public bool IsLocked
        {
            get { return isLocked; }
            set
            {
                isLocked = value;
                if (isLocked)
                {
                    PreviousButton.Enabled = false;
                    NextButton.Enabled = false;
                    FinishButton.Enabled = false;
                }
                else
                    DrawWizardButton(StepType);
            }
        }

        private void InitStepButtons()
        {
            CancelButton.Text = "Cancel";
            CancelButton.Size = new Size(94, 34);
            CancelButton.Location = new Point(30, Height - 30 - CancelButton.Height);
            CancelButton.FlatStyle = FlatStyle.System;
            CancelButton.Click += CancelButton_Click;

            FinishButton.Text = "Finish";
            FinishButton.Size = new Size(94, 34);
            FinishButton.Location = new Point(Width - 30 - FinishButton.Width, Height - 30 - FinishButton.Height);
            FinishButton.FlatStyle = FlatStyle.System;
            FinishButton.Click += FinishButton_Click;

            NextButton.Text = "Next >";
            NextButton.Size = new Size(94, 34);
            NextButton.Location = new Point(FinishButton.Location.X - 30 - NextButton.Width, Height - 30 - FinishButton.Height);
            NextButton.FlatStyle = FlatStyle.System;
            NextButton.Click += NextButton_Click;

            PreviousButton.Text = "< Previous";
            PreviousButton.Size = new Size(94, 34);
            PreviousButton.Location = new Point(NextButton.Location.X - 30 - PreviousButton.Width, Height - 30 - FinishButton.Height);
            PreviousButton.FlatStyle = FlatStyle.System;
            PreviousButton.Click += PreviousButton_Click;

            Controls.Add(PreviousButton);
            Controls.Add(NextButton);
            Controls.Add(FinishButton);
            Controls.Add(CancelButton);
        }

        //public WizardStep(Size size)
        //{
        //    Size = size;
        //    InitStepButtons();
        //    DrawWizardButton(StepType);
        //}

        public WizardStep(string Name)
        {
            base.Name = Name;
            StepType = WizardStepType.Step;
            InitStepButtons();
            DrawWizardButton(StepType);
        }

        public WizardStep(string Name, WizardStepType steptype)
        {
            base.Name = Name;
            StepType = steptype;
            InitStepButtons();
            DrawWizardButton(StepType);
        }

        private void DrawWizardButton(WizardStepType steptype)
        {
            switch(steptype)
            {
                case WizardStepType.Start:
                    {
                        PreviousButton.Enabled = false;
                        NextButton.Enabled = true;
                        FinishButton.Enabled = false;
                        CancelButton.Enabled = true;
                    }
                    break;
                case WizardStepType.Step:
                    {
                        PreviousButton.Enabled = true;
                        NextButton.Enabled = true;
                        FinishButton.Enabled = false;
                        CancelButton.Enabled = true;
                    }
                    break;
                case WizardStepType.Finish :
                    {
                        PreviousButton.Enabled = true;
                        NextButton.Enabled = false;
                        FinishButton.Enabled = true;
                        CancelButton.Enabled = true;
                    }
                    break;
                case WizardStepType.Complete :
                    {
                        PreviousButton.Enabled = false;
                        NextButton.Enabled = false;
                        FinishButton.Enabled = true;
                        CancelButton.Enabled = true;
                    }
                    break;
            }
        }

        protected override void OnResize(EventArgs eventargs)
        {
            base.OnResize(eventargs);

            CancelButton.Location = new Point(30, Height - 30 - CancelButton.Height);
            FinishButton.Location = new Point(Width - 30 - FinishButton.Width, Height - 30 - FinishButton.Height);
            NextButton.Location = new Point(FinishButton.Location.X - 30 - NextButton.Width, Height - 30 - FinishButton.Height);
            PreviousButton.Location = new Point(NextButton.Location.X - 30 - PreviousButton.Width, Height - 30 - FinishButton.Height);
        }

        public delegate void CancelButtonEventHandler(object sender, EventArgs e);
        public delegate void PreviousButtonEventHandler(object sender, EventArgs e);
        public delegate void NextButtonEventHandler(object sender, EventArgs e);
        public delegate void FinishButtonEventHandler(object sender, EventArgs e);

        public event CancelButtonEventHandler OnCancelButtonClick;
        public event PreviousButtonEventHandler OnPreviousButtonClick;
        public event NextButtonEventHandler OnNextButtonClick;
        public event FinishButtonEventHandler OnFinishButtonClick;

        private void CancelButton_Click(object sender, EventArgs e)
        {
            OnCancelButtonClick.Invoke(sender,e);
        }

        private void PreviousButton_Click(object sender,EventArgs e)
        {
            OnPreviousButtonClick.Invoke(sender, e);
        }

        private void NextButton_Click(object sender, EventArgs e)
        {
            if(AllowNext)
                OnNextButtonClick.Invoke(sender, e);
        }

        private void FinishButton_Click(object sender, EventArgs e)
        {
            OnFinishButtonClick.Invoke(sender, e);
        }

    }

    public enum WizardStepType
    {
        Start,
        Step,
        Finish,
        Complete
    }
}